@@ -303,155 +303,71 @@ fn summarize_min_max(
     i: usize,
     stat: &ParquetStatistics,
 ) {
+    if !stat.has_min_max_set() {
+        max_values[i] = None;
+        min_values[i] = None;
+        return;
+    }
     match stat {
-        ParquetStatistics::Boolean(s) => {
-            if let DataType::Boolean = fields[i].data_type() {
-                if s.has_min_max_set() {
-                    if let Some(max_value) = &mut max_values[i] {
-                        match max_value.update_batch(&[Arc::new(BooleanArray::from(
-                            vec![Some(*s.max())],
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                max_values[i] = None;
-                            }
-                        }
-                    }
-                    if let Some(min_value) = &mut min_values[i] {
-                        match min_value.update_batch(&[Arc::new(BooleanArray::from(
-                            vec![Some(*s.min())],
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                min_values[i] = None;
-                            }
-                        }
-                    }
-                    return;
-                }
+        ParquetStatistics::Boolean(s) if DataType::Boolean == *fields[i].data_type() => {
+            if let Some(max_value) = &mut max_values[i] {
+                max_value
+                    .update_batch(&[Arc::new(BooleanArray::from(vec![*s.max()]))])
+                    .unwrap_or_else(|_| max_values[i] = None);
+            }
+            if let Some(min_value) = &mut min_values[i] {
+                min_value
+                    .update_batch(&[Arc::new(BooleanArray::from(vec![*s.min()]))])
+                    .unwrap_or_else(|_| min_values[i] = None);
             }
-            max_values[i] = None;
-            min_values[i] = None;
         }
-        ParquetStatistics::Int32(s) => {
-            if let DataType::Int32 = fields[i].data_type() {
-                if s.has_min_max_set() {
-                    if let Some(max_value) = &mut max_values[i] {
-                        match max_value.update_batch(&[Arc::new(Int32Array::from_value(
-                            *s.max(),
-                            1,
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                max_values[i] = None;
-                            }
-                        }
-                    }
-                    if let Some(min_value) = &mut min_values[i] {
-                        match min_value.update_batch(&[Arc::new(Int32Array::from_value(
-                            *s.min(),
-                            1,
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                min_values[i] = None;
-                            }
-                        }
-                    }
-                    return;
-                }
+        ParquetStatistics::Int32(s) if DataType::Int32 == *fields[i].data_type() => {
+            if let Some(max_value) = &mut max_values[i] {
+                max_value
+                    .update_batch(&[Arc::new(Int32Array::from_value(*s.max(), 1))])
+                    .unwrap_or_else(|_| max_values[i] = None);
+            }
+            if let Some(min_value) = &mut min_values[i] {
+                min_value
+                    .update_batch(&[Arc::new(Int32Array::from_value(*s.min(), 1))])
+                    .unwrap_or_else(|_| min_values[i] = None);
             }
-            max_values[i] = None;
-            min_values[i] = None;
         }
-        ParquetStatistics::Int64(s) => {
-            if let DataType::Int64 = fields[i].data_type() {
-                if s.has_min_max_set() {
-                    if let Some(max_value) = &mut max_values[i] {
-                        match max_value.update_batch(&[Arc::new(Int64Array::from_value(
-                            *s.max(),
-                            1,
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                max_values[i] = None;
-                            }
-                        }
-                    }
-                    if let Some(min_value) = &mut min_values[i] {
-                        match min_value.update_batch(&[Arc::new(Int64Array::from_value(
-                            *s.min(),
-                            1,
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                min_values[i] = None;
-                            }
-                        }
-                    }
-                    return;
-                }
+        ParquetStatistics::Int64(s) if DataType::Int64 == *fields[i].data_type() => {
+            if let Some(max_value) = &mut max_values[i] {
+                max_value
+                    .update_batch(&[Arc::new(Int64Array::from_value(*s.max(), 1))])
+                    .unwrap_or_else(|_| max_values[i] = None);
+            }
+            if let Some(min_value) = &mut min_values[i] {
+                min_value
+                    .update_batch(&[Arc::new(Int64Array::from_value(*s.min(), 1))])
+                    .unwrap_or_else(|_| min_values[i] = None);
             }
-            max_values[i] = None;
-            min_values[i] = None;
         }
-        ParquetStatistics::Float(s) => {
-            if let DataType::Float32 = fields[i].data_type() {
-                if s.has_min_max_set() {
-                    if let Some(max_value) = &mut max_values[i] {
-                        match max_value.update_batch(&[Arc::new(Float32Array::from(
-                            vec![Some(*s.max())],
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                max_values[i] = None;
-                            }
-                        }
-                    }
-                    if let Some(min_value) = &mut min_values[i] {
-                        match min_value.update_batch(&[Arc::new(Float32Array::from(
-                            vec![Some(*s.min())],
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                min_values[i] = None;
-                            }
-                        }
-                    }
-                    return;
-                }
+        ParquetStatistics::Float(s) if DataType::Float32 == *fields[i].data_type() => {
+            if let Some(max_value) = &mut max_values[i] {
+                max_value
+                    .update_batch(&[Arc::new(Float32Array::from(vec![*s.max()]))])
+                    .unwrap_or_else(|_| max_values[i] = None);
+            }
+            if let Some(min_value) = &mut min_values[i] {
+                min_value
+                    .update_batch(&[Arc::new(Float32Array::from(vec![*s.min()]))])
+                    .unwrap_or_else(|_| min_values[i] = None);
             }
-            max_values[i] = None;
-            min_values[i] = None;
         }
-        ParquetStatistics::Double(s) => {
-            if let DataType::Float64 = fields[i].data_type() {
-                if s.has_min_max_set() {
-                    if let Some(max_value) = &mut max_values[i] {
-                        match max_value.update_batch(&[Arc::new(Float64Array::from(
-                            vec![Some(*s.max())],
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                max_values[i] = None;
-                            }
-                        }
-                    }
-                    if let Some(min_value) = &mut min_values[i] {
-                        match min_value.update_batch(&[Arc::new(Float64Array::from(
-                            vec![Some(*s.min())],
-                        ))]) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                min_values[i] = None;
-                            }
-                        }
-                    }
-                    return;
-                }
+        ParquetStatistics::Double(s) if DataType::Float64 == *fields[i].data_type() => {
+            if let Some(max_value) = &mut max_values[i] {
+                max_value
+                    .update_batch(&[Arc::new(Float64Array::from(vec![*s.max()]))])
+                    .unwrap_or_else(|_| max_values[i] = None);
+            }
+            if let Some(min_value) = &mut min_values[i] {
+                min_value
+                    .update_batch(&[Arc::new(Float64Array::from(vec![*s.min()]))])
+                    .unwrap_or_else(|_| min_values[i] = None);
             }
-            max_values[i] = None;
-            min_values[i] = None;
         }
         _ => {
             max_values[i] = None;
@@ -749,7 +665,6 @@ impl DataSink for ParquetSink {
             part_col,
             self.config.table_paths[0].clone(),
             "parquet".into(),
-            self.config.single_file_output,
         );
 
         let mut file_write_tasks: JoinSet<std::result::Result<usize, DataFusionError>> =
@@ -885,16 +800,17 @@ async fn send_arrays_to_col_writers(
     rb: &RecordBatch,
     schema: Arc<Schema>,
 ) -> Result<()> {
-    for (tx, array, field) in col_array_channels
-        .iter()
-        .zip(rb.columns())
-        .zip(schema.fields())
-        .map(|((a, b), c)| (a, b, c))
-    {
+    // Each leaf column has its own channel, increment next_channel for each leaf column sent.
+    let mut next_channel = 0;
+    for (array, field) in rb.columns().iter().zip(schema.fields()) {
         for c in compute_leaves(field, array)? {
-            tx.send(c).await.map_err(|_| {
-                DataFusionError::Internal("Unable to send array to writer!".into())
-            })?;
+            col_array_channels[next_channel]
+                .send(c)
+                .await
+                .map_err(|_| {
+                    DataFusionError::Internal("Unable to send array to writer!".into())
+                })?;
+            next_channel += 1;
         }
     }
 
@@ -902,7 +818,7 @@ async fn send_arrays_to_col_writers(
 }
 
 /// Spawns a tokio task which joins the parallel column writer tasks,
-/// and finalizes the row group.
+/// and finalizes the row group
 fn spawn_rg_join_and_finalize_task(
     column_writer_handles: Vec<JoinHandle<Result<ArrowColumnWriter>>>,
     rg_rows: usize,
