@@ -198,30 +198,45 @@ fn take_optimizable_min(
     stats: &Statistics,
 ) -> Option<(ScalarValue, String)> {
     if let Precision::Exact(num_rows) = &stats.num_rows {
-        if *num_rows > 0 {
-            let col_stats = &stats.column_statistics;
-            if let Some(casted_expr) =
-                agg_expr.as_any().downcast_ref::<expressions::Min>()
-            {
-                if casted_expr.expressions().len() == 1 {
-                    // TODO optimize with exprs other than Column
-                    if let Some(col_expr) = casted_expr.expressions()[0]
-                        .as_any()
-                        .downcast_ref::<expressions::Column>()
+        match *num_rows {
+            0 => {
+                // MIN/MAX with 0 rows is always null
+                if let Some(casted_expr) =
+                    agg_expr.as_any().downcast_ref::<expressions::Min>()
+                {
+                    if let Ok(min_data_type) =
+                        ScalarValue::try_from(casted_expr.field().unwrap().data_type())
                     {
-                        if let Precision::Exact(val) =
-                            &col_stats[col_expr.index()].min_value
+                        return Some((min_data_type, casted_expr.name().to_string()));
+                    }
+                }
+            }
+            value if value > 0 => {
+                let col_stats = &stats.column_statistics;
+                if let Some(casted_expr) =
+                    agg_expr.as_any().downcast_ref::<expressions::Min>()
+                {
+                    if casted_expr.expressions().len() == 1 {
+                        // TODO optimize with exprs other than Column
+                        if let Some(col_expr) = casted_expr.expressions()[0]
+                            .as_any()
+                            .downcast_ref::<expressions::Column>()
                         {
-                            if !val.is_null() {
-                                return Some((
-                                    val.clone(),
-                                    casted_expr.name().to_string(),
-                                ));
+                            if let Precision::Exact(val) =
+                                &col_stats[col_expr.index()].min_value
+                            {
+                                if !val.is_null() {
+                                    return Some((
+                                        val.clone(),
+                                        casted_expr.name().to_string(),
+                                    ));
+                                }
                             }
                         }
                     }
                 }
             }
+            _ => {}
         }
     }
     None
@@ -233,30 +248,45 @@ fn take_optimizable_max(
     stats: &Statistics,
 ) -> Option<(ScalarValue, String)> {
     if let Precision::Exact(num_rows) = &stats.num_rows {
-        if *num_rows > 0 {
-            let col_stats = &stats.column_statistics;
-            if let Some(casted_expr) =
-                agg_expr.as_any().downcast_ref::<expressions::Max>()
-            {
-                if casted_expr.expressions().len() == 1 {
-                    // TODO optimize with exprs other than Column
-                    if let Some(col_expr) = casted_expr.expressions()[0]
-                        .as_any()
-                        .downcast_ref::<expressions::Column>()
+        match *num_rows {
+            0 => {
+                // MIN/MAX with 0 rows is always null
+                if let Some(casted_expr) =
+                    agg_expr.as_any().downcast_ref::<expressions::Max>()
+                {
+                    if let Ok(max_data_type) =
+                        ScalarValue::try_from(casted_expr.field().unwrap().data_type())
                     {
-                        if let Precision::Exact(val) =
-                            &col_stats[col_expr.index()].max_value
+                        return Some((max_data_type, casted_expr.name().to_string()));
+                    }
+                }
+            }
+            value if value > 0 => {
+                let col_stats = &stats.column_statistics;
+                if let Some(casted_expr) =
+                    agg_expr.as_any().downcast_ref::<expressions::Max>()
+                {
+                    if casted_expr.expressions().len() == 1 {
+                        // TODO optimize with exprs other than Column
+                        if let Some(col_expr) = casted_expr.expressions()[0]
+                            .as_any()
+                            .downcast_ref::<expressions::Column>()
                         {
-                            if !val.is_null() {
-                                return Some((
-                                    val.clone(),
-                                    casted_expr.name().to_string(),
-                                ));
+                            if let Precision::Exact(val) =
+                                &col_stats[col_expr.index()].max_value
+                            {
+                                if !val.is_null() {
+                                    return Some((
+                                        val.clone(),
+                                        casted_expr.name().to_string(),
+                                    ));
+                                }
                             }
                         }
                     }
                 }
             }
+            _ => {}
         }
     }
     None
