@@ -91,7 +91,6 @@ impl PhysicalOptimizerRule for CombinePartialFinalAggregate {
                                             input_agg_exec.group_by().clone(),
                                             input_agg_exec.aggr_expr().to_vec(),
                                             input_agg_exec.filter_expr().to_vec(),
-                                            input_agg_exec.order_by_expr().to_vec(),
                                             input_agg_exec.input().clone(),
                                             input_agg_exec.input_schema(),
                                         )
@@ -258,7 +257,6 @@ mod tests {
                 limit: None,
                 table_partition_cols: vec![],
                 output_ordering: vec![],
-                infinite_source: false,
             },
             None,
             None,
@@ -271,13 +269,13 @@ mod tests {
         aggr_expr: Vec<Arc<dyn AggregateExpr>>,
     ) -> Arc<dyn ExecutionPlan> {
         let schema = input.schema();
+        let n_aggr = aggr_expr.len();
         Arc::new(
             AggregateExec::try_new(
                 AggregateMode::Partial,
                 group_by,
                 aggr_expr,
-                vec![],
-                vec![],
+                vec![None; n_aggr],
                 input,
                 schema,
             )
@@ -291,13 +289,13 @@ mod tests {
         aggr_expr: Vec<Arc<dyn AggregateExpr>>,
     ) -> Arc<dyn ExecutionPlan> {
         let schema = input.schema();
+        let n_aggr = aggr_expr.len();
         Arc::new(
             AggregateExec::try_new(
                 AggregateMode::Final,
                 group_by,
                 aggr_expr,
-                vec![],
-                vec![],
+                vec![None; n_aggr],
                 input,
                 schema,
             )
@@ -458,7 +456,6 @@ mod tests {
                 final_group_by,
                 aggr_expr,
                 vec![],
-                vec![],
                 partial_agg,
                 schema,
             )
