@@ -27,30 +27,23 @@ use crate::physical_plan::union::UnionExec;
 use crate::physical_plan::windows::{BoundedWindowAggExec, WindowAggExec};
 use crate::physical_plan::ExecutionPlan;
 
-use datafusion_physical_expr::{LexRequirementRef, PhysicalSortRequirement};
+use datafusion_physical_expr::{LexRequirement, PhysicalSortRequirement};
 use datafusion_physical_plan::limit::{GlobalLimitExec, LocalLimitExec};
+use datafusion_physical_plan::tree_node::PlanContext;
 
 /// This utility function adds a `SortExec` above an operator according to the
 /// given ordering requirements while preserving the original partitioning.
-pub fn add_sort_above(
-    node: &mut Arc<dyn ExecutionPlan>,
-    sort_requirement: LexRequirementRef,
+pub fn add_sort_above<T: Clone + Default>(
+    node: PlanContext<T>,
+    sort_requirements: LexRequirement,
     fetch: Option<usize>,
-) {
-    // If the ordering requirement is already satisfied, do not add a sort.
-    if !node
-        .equivalence_properties()
-        .ordering_satisfy_requirement(sort_requirement)
-    {
-        let sort_expr = PhysicalSortRequirement::to_sort_exprs(sort_requirement.to_vec());
-        let new_sort = SortExec::new(sort_expr, node.clone()).with_fetch(fetch);
-
-        *node = Arc::new(if node.output_partitioning().partition_count() > 1 {
-            new_sort.with_preserve_partitioning(true)
-        } else {
-            new_sort
-        }) as _
+) -> PlanContext<T> {
+    let sort_expr = PhysicalSortRequirement::to_sort_exprs(sort_requirements);
+    let mut new_sort = SortExec::new(sort_expr, node.plan.clone()).with_fetch(fetch);
+    if node.plan.output_partitioning().partition_count() > 1 {
+        new_sort = new_sort.with_preserve_partitioning(true);
     }
+    PlanContext::new(Arc::new(new_sort), T::default(), vec![node])
 }
 
 /// Checks whether the given operator is a limit;
