@@ -263,14 +263,12 @@ impl LogicalPlanBuilder {
         input: LogicalPlan,
         output_url: String,
         file_format: FileType,
-        single_file_output: bool,
         copy_options: CopyOptions,
     ) -> Result<Self> {
         Ok(Self::from(LogicalPlan::Copy(CopyTo {
             input: Arc::new(input),
             output_url,
             file_format,
-            single_file_output,
             copy_options,
         })))
     }
@@ -469,7 +467,7 @@ impl LogicalPlanBuilder {
                         )
                     })
                     .collect::<Result<Vec<_>>>()?;
-                curr_plan.with_new_exprs(curr_plan.expressions(), &new_inputs)
+                curr_plan.with_new_exprs(curr_plan.expressions(), new_inputs)
             }
         }
     }
