@@ -18,7 +18,7 @@
 pub mod context;
 pub mod expr_simplifier;
 mod guarantees;
-mod or_in_list_simplifier;
+mod inlist_simplifier;
 mod regex;
 pub mod simplify_exprs;
 mod utils;
