@@ -25,9 +25,8 @@ use arrow::array::{
     new_null_array, Array, ArrayDataBuilder, ArrayRef, BufferBuilder, GenericStringArray,
     OffsetSizeTrait,
 };
-use arrow_array::builder::{GenericStringBuilder, ListBuilder};
-use arrow_schema::ArrowError;
-use datafusion_common::{arrow_datafusion_err, plan_err};
+
+use datafusion_common::{arrow_datafusion_err, exec_err, plan_err};
 use datafusion_common::{
     cast::as_generic_string_array, internal_err, DataFusionError, Result,
 };
@@ -36,7 +35,9 @@ use hashbrown::HashMap;
 use regex::Regex;
 use std::sync::{Arc, OnceLock};
 
-use crate::functions::{make_scalar_function, make_scalar_function_with_hints, Hint};
+use crate::functions::{
+    make_scalar_function_inner, make_scalar_function_with_hints, Hint,
+};
 
 /// Get the first argument from the given string array.
 ///
@@ -53,25 +54,139 @@ macro_rules! fetch_string_arg {
     }};
 }
 
-/// extract a specific group from a string column, using a regular expression
+/// Tests a string using a regular expression returning true if at
+/// least one match, false otherwise.
+///
+/// The full list of supported features and syntax can be found at
+/// <https://docs.rs/regex/latest/regex/#syntax>
+///
+/// Supported flags can be found at
+/// <https://docs.rs/regex/latest/regex/#grouping-and-flags>
+///
+/// # Examples
+///
+/// ```ignore
+/// # use datafusion::prelude::*;
+/// # use datafusion::error::Result;
+/// # #[tokio::main]
+/// # async fn main() -> Result<()> {
+/// let ctx = SessionContext::new();
+/// let df = ctx.read_csv("tests/data/regex.csv", CsvReadOptions::new()).await?;
+///
+/// // use the regexp_like function to test col 'values',
+/// // against patterns in col 'patterns' without flags
+/// let df = df.with_column(
+///     "a",
+///     regexp_like(vec![col("values"), col("patterns")])
+/// )?;
+/// // use the regexp_like function to test col 'values',
+/// // against patterns in col 'patterns' with flags
+/// let df = df.with_column(
+///     "b",
+///     regexp_like(vec![col("values"), col("patterns"), col("flags")])
+/// )?;
+/// // literals can be used as well with dataframe calls
+/// let df = df.with_column(
+///     "c",
+///     regexp_like(vec![lit("foobarbequebaz"), lit("(bar)(beque)")])
+/// )?;
+///
+/// df.show().await?;
+///
+/// # Ok(())
+/// # }
+/// ```
+pub fn regexp_like<T: OffsetSizeTrait>(args: &[ArrayRef]) -> Result<ArrayRef> {
+    match args.len() {
+        2 => {
+            let values = as_generic_string_array::<T>(&args[0])?;
+            let regex = as_generic_string_array::<T>(&args[1])?;
+            let array = arrow_string::regexp::regexp_is_match_utf8(values, regex, None)
+                .map_err(|e| arrow_datafusion_err!(e))?;
+
+            Ok(Arc::new(array) as ArrayRef)
+        }
+        3 => {
+            let values = as_generic_string_array::<T>(&args[0])?;
+            let regex = as_generic_string_array::<T>(&args[1])?;
+            let flags = as_generic_string_array::<T>(&args[2])?;
+
+            if flags.iter().any(|s| s == Some("g")) {
+                return plan_err!("regexp_like() does not support the \"global\" option");
+            }
+
+            let array = arrow_string::regexp::regexp_is_match_utf8(values, regex, Some(flags))
+                .map_err(|e| arrow_datafusion_err!(e))?;
+
+            Ok(Arc::new(array) as ArrayRef)
+        }
+        other => exec_err!(
+            "regexp_like was called with {other} arguments. It requires at least 2 and at most 3."
+        ),
+    }
+}
+
+/// Extract a specific group from a string column, using a regular expression.
+///
+/// The full list of supported features and syntax can be found at
+/// <https://docs.rs/regex/latest/regex/#syntax>
+///
+/// Supported flags can be found at
+/// <https://docs.rs/regex/latest/regex/#grouping-and-flags>
+///
+/// # Examples
+///
+/// ```ignore
+/// # use datafusion::prelude::*;
+/// # use datafusion::error::Result;
+/// # #[tokio::main]
+/// # async fn main() -> Result<()> {
+/// let ctx = SessionContext::new();
+/// let df = ctx.read_csv("tests/data/regex.csv", CsvReadOptions::new()).await?;
+///
+/// // use  the regexp_match function to test col 'values',
+/// // against patterns in col 'patterns' without flags
+/// let df = df.with_column(
+///     "a",
+///     regexp_match(vec![col("values"), col("patterns")])
+/// )?;
+/// // use the regexp_match function to test col 'values',
+/// // against patterns in col 'patterns' with flags
+/// let df = df.with_column(
+///     "b",
+///     regexp_match(vec![col("values"), col("patterns"), col("flags")]),
+/// )?;
+///
+/// // literals can be used as well with dataframe calls
+/// let df = df.with_column(
+///     "c",
+///     regexp_match(vec![lit("foobarbequebaz"), lit("(bar)(beque)")]),
+/// )?;
+///
+/// df.show().await?;
+///
+/// # Ok(())
+/// # }
+/// ```
 pub fn regexp_match<T: OffsetSizeTrait>(args: &[ArrayRef]) -> Result<ArrayRef> {
     match args.len() {
         2 => {
             let values = as_generic_string_array::<T>(&args[0])?;
             let regex = as_generic_string_array::<T>(&args[1])?;
-            _regexp_match(values, regex, None).map_err(|e| arrow_datafusion_err!(e))
+            arrow_string::regexp::regexp_match(values, regex, None)
+                .map_err(|e| arrow_datafusion_err!(e))
         }
         3 => {
             let values = as_generic_string_array::<T>(&args[0])?;
             let regex = as_generic_string_array::<T>(&args[1])?;
-            let flags = Some(as_generic_string_array::<T>(&args[2])?);
+            let flags = as_generic_string_array::<T>(&args[2])?;
 
-            match flags {
-                Some(f) if f.iter().any(|s| s == Some("g")) => {
-                    plan_err!("regexp_match() does not support the \"global\" option")
-                },
-                _ => _regexp_match(values, regex, flags).map_err(|e| arrow_datafusion_err!(e)),
+            if flags.iter().any(|s| s == Some("g")) {
+                return plan_err!("regexp_match() does not support the \"global\" option")
             }
+
+            arrow_string::regexp::regexp_match(values, regex, Some(flags))
+                .map_err(|e| arrow_datafusion_err!(e))
         }
         other => internal_err!(
             "regexp_match was called with {other} arguments. It requires at least 2 and at most 3."
@@ -79,83 +194,6 @@ pub fn regexp_match<T: OffsetSizeTrait>(args: &[ArrayRef]) -> Result<ArrayRef> {
     }
 }
 
-/// TODO: Remove this once it is included in arrow-rs new release.
-/// <https://github.com/apache/arrow-rs/pull/5235>
-fn _regexp_match<OffsetSize: OffsetSizeTrait>(
-    array: &GenericStringArray<OffsetSize>,
-    regex_array: &GenericStringArray<OffsetSize>,
-    flags_array: Option<&GenericStringArray<OffsetSize>>,
-) -> std::result::Result<ArrayRef, ArrowError> {
-    let mut patterns: std::collections::HashMap<String, Regex> =
-        std::collections::HashMap::new();
-    let builder: GenericStringBuilder<OffsetSize> =
-        GenericStringBuilder::with_capacity(0, 0);
-    let mut list_builder = ListBuilder::new(builder);
-
-    let complete_pattern = match flags_array {
-        Some(flags) => Box::new(regex_array.iter().zip(flags.iter()).map(
-            |(pattern, flags)| {
-                pattern.map(|pattern| match flags {
-                    Some(value) => format!("(?{value}){pattern}"),
-                    None => pattern.to_string(),
-                })
-            },
-        )) as Box<dyn Iterator<Item = Option<String>>>,
-        None => Box::new(
-            regex_array
-                .iter()
-                .map(|pattern| pattern.map(|pattern| pattern.to_string())),
-        ),
-    };
-
-    array
-        .iter()
-        .zip(complete_pattern)
-        .map(|(value, pattern)| {
-            match (value, pattern) {
-                // Required for Postgres compatibility:
-                // SELECT regexp_match('foobarbequebaz', ''); = {""}
-                (Some(_), Some(pattern)) if pattern == *"" => {
-                    list_builder.values().append_value("");
-                    list_builder.append(true);
-                }
-                (Some(value), Some(pattern)) => {
-                    let existing_pattern = patterns.get(&pattern);
-                    let re = match existing_pattern {
-                        Some(re) => re,
-                        None => {
-                            let re = Regex::new(pattern.as_str()).map_err(|e| {
-                                ArrowError::ComputeError(format!(
-                                    "Regular expression did not compile: {e:?}"
-                                ))
-                            })?;
-                            patterns.insert(pattern.clone(), re);
-                            patterns.get(&pattern).unwrap()
-                        }
-                    };
-                    match re.captures(value) {
-                        Some(caps) => {
-                            let mut iter = caps.iter();
-                            if caps.len() > 1 {
-                                iter.next();
-                            }
-                            for m in iter.flatten() {
-                                list_builder.values().append_value(m.as_str());
-                            }
-
-                            list_builder.append(true);
-                        }
-                        None => list_builder.append(false),
-                    }
-                }
-                _ => list_builder.append(false),
-            }
-            Ok(())
-        })
-        .collect::<std::result::Result<Vec<()>, ArrowError>>()?;
-    Ok(Arc::new(list_builder.finish()))
-}
-
 /// replace POSIX capture groups (like \1) with Rust Regex group (like ${1})
 /// used by regexp_replace
 fn regex_replace_posix_groups(replacement: &str) -> String {
@@ -168,9 +206,46 @@ fn regex_replace_posix_groups(replacement: &str) -> String {
         .into_owned()
 }
 
-/// Replaces substring(s) matching a POSIX regular expression.
+/// Replaces substring(s) matching a PCRE-like regular expression.
 ///
-/// example: `regexp_replace('Thomas', '.[mN]a.', 'M') = 'ThM'`
+/// The full list of supported features and syntax can be found at
+/// <https://docs.rs/regex/latest/regex/#syntax>
+///
+/// Supported flags with the addition of 'g' can be found at
+/// <https://docs.rs/regex/latest/regex/#grouping-and-flags>
+///
+/// # Examples
+///
+/// ```ignore
+/// # use datafusion::prelude::*;
+/// # use datafusion::error::Result;
+/// # #[tokio::main]
+/// # async fn main() -> Result<()> {
+/// let ctx = SessionContext::new();
+/// let df = ctx.read_csv("tests/data/regex.csv", CsvReadOptions::new()).await?;
+///
+/// // use the regexp_replace function to replace substring(s) without flags
+/// let df = df.with_column(
+///     "a",
+///     regexp_replace(vec![col("values"), col("patterns"), col("replacement")])
+/// )?;
+/// // use the regexp_replace function to replace substring(s) with flags
+/// let df = df.with_column(
+///     "b",
+///     regexp_replace(vec![col("values"), col("patterns"), col("replacement"), col("flags")]),
+/// )?;
+///
+/// // literals can be used as well
+/// let df = df.with_column(
+///     "c",
+///     regexp_replace(vec![lit("foobarbequebaz"), lit("(bar)(beque)"), lit(r"\2")]),
+/// )?;
+///
+/// df.show().await?;
+///
+/// # Ok(())
+/// # }
+/// ```
 pub fn regexp_replace<T: OffsetSizeTrait>(args: &[ArrayRef]) -> Result<ArrayRef> {
     // Default implementation for regexp_replace, assumes all args are arrays
     // and args is a sequence of 3 or 4 elements.
@@ -192,7 +267,7 @@ pub fn regexp_replace<T: OffsetSizeTrait>(args: &[ArrayRef]) -> Result<ArrayRef>
                 (Some(string), Some(pattern), Some(replacement)) => {
                     let replacement = regex_replace_posix_groups(replacement);
 
-                    // if patterns hashmap already has regexp then use else else create and return
+                    // if patterns hashmap already has regexp then use else create and return
                     let re = match patterns.get(pattern) {
                         Some(re) => Ok(re),
                         None => {
@@ -238,7 +313,7 @@ pub fn regexp_replace<T: OffsetSizeTrait>(args: &[ArrayRef]) -> Result<ArrayRef>
                         (format!("(?{flags}){pattern}"), false)
                     };
 
-                    // if patterns hashmap already has regexp then use else else create and return
+                    // if patterns hashmap already has regexp then use else create and return
                     let re = match patterns.get(&pattern) {
                         Some(re) => Ok(re),
                         None => {
@@ -276,13 +351,13 @@ fn _regexp_replace_early_abort<T: OffsetSizeTrait>(
     input_array: &GenericStringArray<T>,
 ) -> Result<ArrayRef> {
     // Mimicking the existing behavior of regexp_replace, if any of the scalar arguments
-    // are actuall null, then the result will be an array of the same size but with nulls.
+    // are actually null, then the result will be an array of the same size but with nulls.
     //
     // Also acts like an early abort mechanism when the input array is empty.
     Ok(new_null_array(input_array.data_type(), input_array.len()))
 }
 
-/// Special cased regex_replace implementation for the scenerio where
+/// Special cased regex_replace implementation for the scenario where
 /// the pattern, replacement and flags are static (arrays that are derived
 /// from scalars). This means we can skip regex caching system and basically
 /// hold a single Regex object for the replace operation. This also speeds
@@ -401,16 +476,76 @@ pub fn specialize_regexp_replace<T: OffsetSizeTrait>(
 
         // If there are no specialized implementations, we'll fall back to the
         // generic implementation.
-        (_, _, _, _) => Ok(make_scalar_function(regexp_replace::<T>)),
+        (_, _, _, _) => Ok(make_scalar_function_inner(regexp_replace::<T>)),
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use arrow::array::*;
+
     use datafusion_common::ScalarValue;
 
+    use super::*;
+
+    #[test]
+    fn test_case_sensitive_regexp_like() {
+        let values = StringArray::from(vec!["abc"; 5]);
+
+        let patterns =
+            StringArray::from(vec!["^(a)", "^(A)", "(b|d)", "(B|D)", "^(b|c)"]);
+
+        let mut expected_builder: BooleanBuilder = BooleanBuilder::new();
+        expected_builder.append_value(true);
+        expected_builder.append_value(false);
+        expected_builder.append_value(true);
+        expected_builder.append_value(false);
+        expected_builder.append_value(false);
+        let expected = expected_builder.finish();
+
+        let re = regexp_like::<i32>(&[Arc::new(values), Arc::new(patterns)]).unwrap();
+
+        assert_eq!(re.as_ref(), &expected);
+    }
+
+    #[test]
+    fn test_case_insensitive_regexp_like() {
+        let values = StringArray::from(vec!["abc"; 5]);
+        let patterns =
+            StringArray::from(vec!["^(a)", "^(A)", "(b|d)", "(B|D)", "^(b|c)"]);
+        let flags = StringArray::from(vec!["i"; 5]);
+
+        let mut expected_builder: BooleanBuilder = BooleanBuilder::new();
+        expected_builder.append_value(true);
+        expected_builder.append_value(true);
+        expected_builder.append_value(true);
+        expected_builder.append_value(true);
+        expected_builder.append_value(false);
+        let expected = expected_builder.finish();
+
+        let re =
+            regexp_like::<i32>(&[Arc::new(values), Arc::new(patterns), Arc::new(flags)])
+                .unwrap();
+
+        assert_eq!(re.as_ref(), &expected);
+    }
+
+    #[test]
+    fn test_unsupported_global_flag_regexp_like() {
+        let values = StringArray::from(vec!["abc"]);
+        let patterns = StringArray::from(vec!["^(a)"]);
+        let flags = StringArray::from(vec!["g"]);
+
+        let re_err =
+            regexp_like::<i32>(&[Arc::new(values), Arc::new(patterns), Arc::new(flags)])
+                .expect_err("unsupported flag should have failed");
+
+        assert_eq!(
+            re_err.strip_backtrace(),
+            "Error during planning: regexp_like() does not support the \"global\" option"
+        );
+    }
+
     #[test]
     fn test_case_sensitive_regexp_match() {
         let values = StringArray::from(vec!["abc"; 5]);
